"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CloudEnabledLifeCycle = void 0;
const path = require("path");
const path_1 = require("path");
const fs_1 = require("fs");
const environment_1 = require("../../../utilities/environment");
const serializer_overrides_1 = require("../../../utilities/serializer-overrides");
const { workspaceRoot } = require('../../../utilities/nx-imports');
class CloudEnabledLifeCycle {
    constructor(runContext, cacheDirectory, skipNxCache, collectTerminalOutput, cacheableOperations, outputObfuscator, tasks) {
        this.runContext = runContext;
        this.cacheDirectory = cacheDirectory;
        this.skipNxCache = skipNxCache;
        this.collectTerminalOutput = collectTerminalOutput;
        this.cacheableOperations = cacheableOperations;
        this.outputObfuscator = outputObfuscator;
        this.tasks = tasks;
    }
    scheduleTask(task) {
        this.runContext.scheduledTasks.push(task);
    }
    startTask(task) {
        this.tasks.push({
            taskId: task.id,
            startTime: new Date().toISOString(),
            target: task.target.target,
            projectName: task.target.project,
            hash: task.hash,
            hashDetails: this.cleanUpHashDetails(task.hashDetails),
            params: (0, serializer_overrides_1.serializeOverrides)(task),
        });
    }
    // Used for older version for Nx. Remove in Dec 2022
    endTask(task, code) {
        let cacheStatus;
        if (this.runContext.statuses[task.hash]) {
            cacheStatus = this.runContext.statuses[task.hash];
        }
        else {
            if (this.cacheableOperations.indexOf(task.target.target) > -1 &&
                !this.skipNxCache) {
                cacheStatus = 'local-cache-hit';
            }
            else {
                cacheStatus = 'cache-miss';
            }
        }
        this.updateStartedTask(task, code, cacheStatus);
    }
    endTasks(tasks) {
        for (let t of tasks) {
            let cacheStatus;
            const taskIsRemoteCacheHit = t.status === 'remote-cache';
            const taskIsLocalCacheHitPreNx135 = t.status === 'cache';
            const taskIsLocalCacheHit = t.status === 'local-cache' ||
                t.status === 'local-cache-kept-existing' ||
                taskIsLocalCacheHitPreNx135;
            if (this.runContext.statuses[t.task.hash]) {
                cacheStatus = this.runContext.statuses[t.task.hash];
            }
            else if (taskIsRemoteCacheHit) {
                cacheStatus = 'remote-cache-hit';
            }
            else if (taskIsLocalCacheHit) {
                cacheStatus = 'local-cache-hit';
            }
            else {
                if (this.cacheableOperations.indexOf(t.task.target.target) > -1 &&
                    !this.skipNxCache) {
                    cacheStatus = 'local-cache-hit';
                }
                else {
                    cacheStatus = 'cache-miss';
                }
            }
            this.updateStartedTask(t.task, t.code, cacheStatus);
        }
    }
    endCommand() { }
    updateStartedTask(task, code, cacheStatus) {
        const startedTask = this.tasks.find((t) => t.taskId === task.id);
        if (!startedTask) {
            throw new Error(`Cannot find task ${task.id}`);
        }
        startedTask.endTime = new Date().toISOString();
        startedTask.status = code;
        startedTask.params = this.outputObfuscator.obfuscate(startedTask.params);
        startedTask.cacheStatus = cacheStatus;
        if (this.collectTerminalOutput) {
            startedTask.terminalOutput = this.getTerminalOutput(task.hash, startedTask.cacheStatus, code);
        }
    }
    getTerminalOutput(hash, cacheStatus, code) {
        let dir;
        if (this.cacheDirectory) {
            if (this.cacheDirectory.startsWith('./')) {
                dir = (0, path_1.join)(workspaceRoot, this.cacheDirectory);
            }
            else {
                dir = this.cacheDirectory;
            }
        }
        else {
            dir = (0, path_1.join)(workspaceRoot, 'node_modules', '.cache', 'nx');
        }
        try {
            const taskOutput = this.readTerminalOutputFile(dir, hash);
            const taskOutputSanitized = this.outputObfuscator.obfuscate(taskOutput);
            if (environment_1.NX_CLOUD_UNLIMITED_OUTPUT)
                return taskOutputSanitized;
            const maxCharacters = cacheStatus === 'cache-miss'
                ? code === 0
                    ? CloudEnabledLifeCycle.MAX_CHARACTERS_SUCCESSFUL_TASK
                    : CloudEnabledLifeCycle.MAX_CHARACTERS_FAILED_TASK
                : CloudEnabledLifeCycle.MAX_CHARACTERS_CACHED_TASK;
            return taskOutputSanitized.length > maxCharacters
                ? `TRUNCATED\n\n${taskOutputSanitized.slice(taskOutputSanitized.length - maxCharacters)}`
                : taskOutputSanitized;
        }
        catch (e) {
            return '';
        }
    }
    cleanUpHashDetails(hashDetails) {
        const nodes = {};
        Object.keys(hashDetails.nodes)
            .filter((k) => !k.startsWith('npm:'))
            .forEach((k) => {
            nodes[k] = hashDetails.nodes[k];
        });
        return {
            nodes,
            runtime: hashDetails.runtime,
            implicitDeps: hashDetails.implicitDeps,
        };
    }
    readTerminalOutputFile(dir, hash) {
        try {
            return (0, fs_1.readFileSync)(path.join(dir, 'terminalOutputs', hash)).toString();
        }
        catch (_a) {
            try {
                return (0, fs_1.readFileSync)(path.join(dir, hash, 'terminalOutput')).toString();
            }
            catch (_b) {
                return '';
            }
        }
    }
}
exports.CloudEnabledLifeCycle = CloudEnabledLifeCycle;
CloudEnabledLifeCycle.MAX_CHARACTERS_FAILED_TASK = 200000;
CloudEnabledLifeCycle.MAX_CHARACTERS_SUCCESSFUL_TASK = 20000;
CloudEnabledLifeCycle.MAX_CHARACTERS_CACHED_TASK = 20000;
//# sourceMappingURL=cloud-enabled-life-cycle.js.map