"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cloudEnabledTasksRunner = void 0;
const message_reporter_1 = require("../../terminal-output/message-reporter");
const end_of_run_message_1 = require("../../terminal-output/end-of-run-message");
const output_obfuscator_1 = require("../../terminal-output/output-obfuscator");
const cloud_enabled_life_cycle_1 = require("./cloud-enabled-life-cycle");
const file_storage_1 = require("../../file-storage/file-storage");
const e2e_encryption_1 = require("../../file-storage/e2e-encryption");
const environment_1 = require("../../../utilities/environment");
const cloud_remote_cache_1 = require("./cloud-remote-cache");
const cloud_run_api_1 = require("./cloud-run.api");
const fs_1 = require("fs");
const path = require("path");
const metric_logger_1 = require("../../../utilities/metric-logger");
const error_reporter_api_1 = require("../../api/error-reporter.api");
const path_1 = require("path");
const fs_extra_1 = require("fs-extra");
const { tasksRunner, output, Cache } = require('../../../utilities/nx-imports');
function createApi(errors, options, runContext) {
    const machineInfo = (0, environment_1.getMachineInfo)(options);
    return new cloud_run_api_1.CloudRunApi(errors, runContext, options, machineInfo);
}
function storeTaskHashes(taskExecutions, directory, distributedExecutionId) {
    const hashes = JSON.stringify(taskExecutions.map((t) => ({
        taskId: t.taskId,
        hash: t.hash,
    })));
    if (environment_1.VERBOSE_LOGGING) {
        output.note({
            title: `Executed tasks with hashes: ${hashes}`,
        });
    }
    (0, fs_1.writeFileSync)(path.join(directory, `tasks-hashes-${distributedExecutionId}`), hashes);
}
function onComplete(lifeCycle, options, remoteCache, api, outputObfuscator, runStartTime, messages, endOfRunMessage, taskExecutions, versionOfNxBefore133, inner) {
    return __awaiter(this, void 0, void 0, function* () {
        const runEndTime = new Date().toISOString();
        const branch = (0, environment_1.getBranch)();
        const runData = {
            command: outputObfuscator.obfuscate((0, environment_1.parseCommand)()),
            startTime: runStartTime,
            endTime: runEndTime,
            distributedExecutionId: environment_1.NX_CLOUD_DISTRIBUTED_EXECUTION_ID,
            branch: branch,
            scan: true,
            runGroup: (0, environment_1.getRunGroup)(),
            // only set sha if branch is set because we invoke a separate process,
            // which adds a few millis
            sha: branch ? (0, environment_1.extractGitSha)() : undefined,
            inner: inner,
        };
        if (environment_1.AGENT_RUNNING_IN_DISTRIBUTED_EXECUTION) {
            storeTaskHashes(taskExecutions, options.cacheDirectory || './node_modules/.cache/nx', environment_1.NX_CLOUD_DISTRIBUTED_EXECUTION_ID);
            try {
                yield remoteCache.waitForStoreRequestsToComplete();
            }
            catch (e) {
                output.error({
                    title: `Agent wasn't able to store artifacts`,
                });
                messages.printMessages();
                process.exit(environment_1.DISTRIBUTED_TASK_EXECUTION_INTERNAL_ERROR_STATUS_CODE);
            }
            try {
                yield api.endRun(runData, taskExecutions);
            }
            catch (e) {
                output.error({
                    title: `Agent wasn't able to record its run`,
                });
                messages.printMessages();
                process.exit(environment_1.DISTRIBUTED_TASK_EXECUTION_INTERNAL_ERROR_STATUS_CODE);
            }
        }
        else {
            yield Promise.all([
                remoteCache.waitForStoreRequestsToComplete(),
                api.endRun(runData, taskExecutions),
            ]);
        }
        // this workaround is required because prior to Nx 13.3 we printed
        // the end message after the runner completes, so we need to wait for the
        // message to appear in the right place
        if (versionOfNxBefore133) {
            setTimeout(() => {
                messages.printMessages();
                if (!messages.anyErrors && !inner) {
                    endOfRunMessage.printCacheHitsMessage();
                }
            }, 0);
        }
        else {
            messages.printMessages();
            if (!messages.anyErrors && !inner) {
                endOfRunMessage.printCacheHitsMessage();
            }
        }
        yield (0, metric_logger_1.submitRunMetrics)(options);
    });
}
function createLifeCycle(runContext, options, outputObfuscator, tasks) {
    const cloudEnabledLifeCycle = new cloud_enabled_life_cycle_1.CloudEnabledLifeCycle(runContext, options.cacheDirectory, !!options.skipNxCache, options.scan === undefined ? true : options.scan, options.cacheableOperations || [], outputObfuscator, tasks);
    try {
        const { CompositeLifeCycle } = require('../../../utilities/nx-imports');
        if (!CompositeLifeCycle)
            return cloudEnabledLifeCycle;
        return new CompositeLifeCycle([options.lifeCycle, cloudEnabledLifeCycle]);
    }
    catch (e) {
        return cloudEnabledLifeCycle;
    }
}
function fetchUrlsForKnownHashesUpfront(api, runContext, tasks, options) {
    return __awaiter(this, void 0, void 0, function* () {
        let hashes = tasks.map((t) => t.hash).filter((h) => !!h);
        const c = new Cache(options);
        if (!options.skipNxCache) {
            // filter out the urls that don't have artifact stored locally
            const promises = yield Promise.all(hashes.map((hash) => {
                const tdCommit = (0, path_1.join)(c.cachePath, `${hash}.commit`);
                return (0, fs_extra_1.pathExists)(tdCommit);
            }));
            const missingHashes = [];
            for (let i = 0; i < promises.length; ++i) {
                if (promises[i]) {
                    missingHashes.push(hashes[i]);
                }
            }
            hashes = missingHashes;
        }
        if (hashes.length > 0) {
            const startRunRequest = api.startRun(environment_1.NX_CLOUD_DISTRIBUTED_EXECUTION_ID, hashes);
            for (const hash of hashes) {
                runContext.requests[hash] = startRunRequest;
            }
        }
    });
}
function cloudEnabledTasksRunner(tasks, options, context, inner = false) {
    const runContext = {
        statuses: {},
        scheduledTasks: [],
        requests: {},
        allTasks: tasks,
    };
    const versionOfNxBefore133 = options.lifeCycle === undefined;
    const taskExecutions = [];
    const messages = new message_reporter_1.MessageReporter(options);
    const api = createApi(messages, options, runContext);
    const endOfRunMessage = new end_of_run_message_1.EndOfRunMessage(runContext, taskExecutions);
    const outputObfuscator = new output_obfuscator_1.OutputObfuscator(options.maskedProperties);
    const runStartTime = new Date().toISOString();
    const lifeCycle = createLifeCycle(runContext, options, outputObfuscator, taskExecutions);
    const encryption = new e2e_encryption_1.E2EEncryption(environment_1.ENCRYPTION_KEY || options.encryptionKey);
    const errorApi = new error_reporter_api_1.ErrorReporterApi(options);
    const fileStorage = new file_storage_1.FileStorage(encryption, errorApi, false);
    const remoteCache = new cloud_remote_cache_1.CloudRemoteCache(messages, api, runContext, fileStorage);
    // we don't await for it cause this step is optional
    fetchUrlsForKnownHashesUpfront(api, runContext, tasks, options);
    // have to reset it so we don't capture inner tasks
    delete process.env.NX_CLOUD_DISTRIBUTED_EXECUTION_ID;
    const res = tasksRunner(tasks, Object.assign(Object.assign({}, options), { remoteCache, lifeCycle }), context);
    // observable -> legacy
    if (res.subscribe) {
        const { Subject } = require('rxjs/internal/Subject');
        const wrappedRes = new Subject();
        res.subscribe({
            next: (value) => wrappedRes.next(value),
            error: (err) => wrappedRes.error(err),
            complete: () => __awaiter(this, void 0, void 0, function* () {
                yield onComplete(lifeCycle, options, remoteCache, api, outputObfuscator, runStartTime, messages, endOfRunMessage, taskExecutions, versionOfNxBefore133, inner);
                wrappedRes.complete();
            }),
        });
        return wrappedRes;
        // promise
    }
    else {
        return res
            .then((r) => __awaiter(this, void 0, void 0, function* () {
            yield onComplete(lifeCycle, options, remoteCache, api, outputObfuscator, runStartTime, messages, endOfRunMessage, taskExecutions, versionOfNxBefore133, inner);
            return r;
        }))
            .catch((e) => __awaiter(this, void 0, void 0, function* () {
            yield onComplete(lifeCycle, options, remoteCache, api, outputObfuscator, runStartTime, messages, endOfRunMessage, taskExecutions, versionOfNxBefore133, inner);
            throw e;
        }));
    }
}
exports.cloudEnabledTasksRunner = cloudEnabledTasksRunner;
//# sourceMappingURL=cloud-enabled.runner.js.map