"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startAgent = void 0;
const child_process_1 = require("child_process");
const fs_1 = require("fs");
const stripJsonComments = require("strip-json-comments");
const create_unchanged_value_timeout_1 = require("../../../utilities/create-unchanged-value-timeout");
const environment_1 = require("../../../utilities/environment");
const metric_logger_1 = require("../../../utilities/metric-logger");
const waiter_1 = require("../../../utilities/waiter");
const print_run_group_error_1 = require("../../error/print-run-group-error");
const distributed_agent_api_1 = require("./distributed-agent.api");
const { output, workspaceRoot } = require('../../../utilities/nx-imports');
function executeTasks(options, api) {
    return __awaiter(this, void 0, void 0, function* () {
        let completedStatusCode = 0;
        let apiResponse = null;
        const failIfSameTasksAfterTimeout = (0, create_unchanged_value_timeout_1.createUnchangedValueTimeout)({
            title: `No new messages received after ${environment_1.NO_MESSAGES_TIMEOUT / 1000} seconds`,
            timeout: environment_1.NO_MESSAGES_TIMEOUT,
        });
        const waiter = new waiter_1.Waiter();
        let completedTasks = [];
        const startTime = new Date();
        let executedAnyTasks = false;
        while (true) {
            if (environment_1.VERBOSE_LOGGING) {
                output.note({
                    title: 'Fetching tasks...',
                });
            }
            apiResponse = yield api.tasks(apiResponse ? apiResponse.executionId : null, completedStatusCode, completedTasks);
            if (environment_1.VERBOSE_LOGGING) {
                output.note({
                    title: 'API Response',
                    bodyLines: [
                        `completed: ${apiResponse.completed}`,
                        `status: ${apiResponse.status}`,
                        `retryDuring: ${apiResponse.retryDuring}`,
                        `executionId: ${apiResponse.executionId}`,
                        `number of tasks: ${apiResponse.tasks.length}`,
                        `error: ${apiResponse.criticalErrorMessage}`,
                        `maxParallel: ${apiResponse.maxParallel}`,
                    ],
                });
            }
            if (apiResponse.criticalErrorMessage) {
                output.error({
                    title: 'Distributed Execution Terminated',
                    bodyLines: ['Error:', apiResponse.criticalErrorMessage],
                });
                process.exit(0);
            }
            // run group is completed but it might be a rerun
            // we will try several times before going further and
            // completed the response
            // we only do it if we haven't executed any tasks
            if ((apiResponse === null || apiResponse === void 0 ? void 0 : apiResponse.retryDuring) &&
                (apiResponse === null || apiResponse === void 0 ? void 0 : apiResponse.retryDuring) !== 0 &&
                !executedAnyTasks &&
                new Date().getTime() - startTime.getTime() > apiResponse.retryDuring) {
                yield waiter.wait();
                continue;
            }
            if ((apiResponse === null || apiResponse === void 0 ? void 0 : apiResponse.status) !== undefined) {
                if (apiResponse.status === 'RUN_GROUP_COMPLETED' ||
                    apiResponse.status === 'NO_FURTHER_TASKS_TO_RUN') {
                    return;
                }
            }
            else if (apiResponse.completed) {
                return;
            }
            // if status is present that use the status instead of completed, otherwise use completed
            failIfSameTasksAfterTimeout(apiResponse.tasks.map((t) => t.taskId).join(''));
            if (!apiResponse.executionId) {
                if (environment_1.VERBOSE_LOGGING) {
                    output.note({
                        title: 'Waiting...',
                    });
                }
                yield waiter.wait();
                completedStatusCode = 0;
                completedTasks = [];
                continue;
            }
            waiter.reset();
            executedAnyTasks = true;
            const r = invokeTasksUsingRunMany(options, apiResponse.executionId, apiResponse.tasks, apiResponse.maxParallel);
            completedStatusCode = r.completedStatusCode;
            completedTasks = r.completedTasks;
        }
    });
}
function readCompletedTasks(options, distributedExecutionId) {
    const errorMessage = `Command execution failed (distributed task execution: ${distributedExecutionId}). Tasks hashes haven\'t been recorded.`;
    let completedTasks;
    try {
        const cacheDirectory = options.cacheDirectory || './node_modules/.cache/nx';
        const taskHashesFile = `${cacheDirectory}/tasks-hashes-${distributedExecutionId}`;
        completedTasks = JSON.parse((0, fs_1.readFileSync)(taskHashesFile).toString());
        // remove it such that if the next command crashes we don't read an obsolete file
        (0, fs_1.unlinkSync)(taskHashesFile);
    }
    catch (e) {
        throw new Error(errorMessage);
    }
    if (completedTasks.length == 0) {
        throw new Error(errorMessage);
    }
    return completedTasks;
}
function invokeTasksUsingRunMany(options, executionId, tasks, maxParallel) {
    let completedStatusCode = 0;
    const completedTasks = [];
    for (const g of groupByTarget(tasks)) {
        const config = g.configuration ? `--configuration=${g.configuration}` : ``;
        const parallel = maxParallel > 1 ? ` --parallel --max-parallel=${maxParallel}` : ``;
        // TODO use pnpx or yarn when needed
        const command = `npx nx run-many --target=${g.target} ${config} --projects=${g.projects.join(',')} ${g.params}${parallel}`;
        if (environment_1.VERBOSE_LOGGING) {
            output.note({
                title: `Executing: '${command}'`,
            });
        }
        try {
            (0, child_process_1.execSync)(command, {
                stdio: ['inherit', 'inherit', 'inherit'],
                env: Object.assign(Object.assign({}, process.env), { NX_CACHE_FAILURES: 'true', NX_CLOUD_DISTRIBUTED_EXECUTION_ID: executionId }),
            });
            completedTasks.push(...readCompletedTasks(options, executionId));
        }
        catch (e) {
            if (e.status === environment_1.DISTRIBUTED_TASK_EXECUTION_INTERNAL_ERROR_STATUS_CODE) {
                throw e;
            }
            else {
                completedStatusCode = 1;
                completedTasks.push(...readCompletedTasks(options, executionId));
            }
        }
    }
    return { completedStatusCode, completedTasks };
}
function groupByTarget(tasks) {
    const res = [];
    tasks.forEach((t) => {
        const r = res.find((rr) => rr.target === t.target && rr.configuration === t.configuration);
        if (r) {
            r.projects.push(t.projectName);
        }
        else {
            res.push({
                target: t.target,
                projects: [t.projectName],
                params: t.params,
                configuration: t.configuration,
            });
        }
    });
    return res;
}
function getAgentName() {
    if (process.env.NX_AGENT_NAME !== undefined) {
        return process.env.NX_AGENT_NAME;
    }
    else if (process.env.CIRCLECI !== undefined && process.env.CIRCLE_STAGE) {
        return process.env.CIRCLE_STAGE;
    }
    else if (process.env.CIRCLECI !== undefined && process.env.CIRCLE_JOB) {
        return process.env.CIRCLE_JOB;
    }
    else {
        return `Agent ${Math.floor(Math.random() * 100000)}`;
    }
}
function createAgentLockfileAndSetUpListeners(api, options, agentName) {
    const cacheDirectory = options.cacheDirectory || './node_modules/.cache/nx';
    const lockFileDirectory = `${cacheDirectory}/lockfiles`;
    const lockFilePath = `${lockFileDirectory}/${agentName}.lock`;
    if (!(0, fs_1.existsSync)(lockFileDirectory)) {
        (0, fs_1.mkdirSync)(lockFileDirectory, { recursive: true });
    }
    // Check for other agents' lockfiles and warn if exist
    const lockFiles = (0, fs_1.readdirSync)(lockFileDirectory);
    if (lockFiles.length) {
        // Check to make sure the current agent name is not in use (only 1/100000 ^ 2 chance of this)
        if (lockFiles.includes(`${agentName}.lock`)) {
            output.error({
                title: 'Duplicate Agent ID Detected',
                bodyLines: [
                    'We have detected another agent with this ID running in this workspace. This should not happen.',
                    '',
                    'End all currently running agents, run "npx nx-cloud clean-up-agents", and try again.',
                ],
            });
            process.exit(1);
        }
        output.warn({
            title: 'Other Nx Cloud Agents Detected',
            bodyLines: [
                'We have detected other agents running in this workspace. This can cause unexpected behavior.',
                '',
                'This can also be a false positive caused by agents that did not shut down correctly.',
                'If you believe this is the case, run "npx nx-cloud clean-up-agents".',
            ],
        });
    }
    (0, fs_1.writeFileSync)(lockFilePath, '');
    process.on('exit', (code) => {
        cleanupAgentLockfile(lockFilePath, code);
    });
    process.on('SIGTERM', () => __awaiter(this, void 0, void 0, function* () {
        yield api.completeRunGroupWithError('Agent was terminated via SIGTERM');
        cleanupAgentLockfile(lockFilePath, 1);
    }));
    process.on('SIGINT', () => __awaiter(this, void 0, void 0, function* () {
        yield api.completeRunGroupWithError('Agent was terminated via SIGINT');
        cleanupAgentLockfile(lockFilePath, 1);
    }));
}
function cleanupAgentLockfile(lockFilePath, code) {
    if ((0, fs_1.existsSync)(lockFilePath)) {
        (0, fs_1.unlinkSync)(lockFilePath);
        process.exit(code);
    }
}
function startAgent() {
    return __awaiter(this, void 0, void 0, function* () {
        const runGroup = (0, environment_1.getRunGroup)();
        if (!runGroup) {
            (0, print_run_group_error_1.printRunGroupError)();
            return process.exit(1);
        }
        output.note({
            title: 'Starting an agent for running Nx tasks',
        });
        const options = JSON.parse(stripJsonComments((0, fs_1.readFileSync)(`${workspaceRoot}/nx.json`).toString())).tasksRunnerOptions.default.options;
        const agentName = getAgentName();
        const api = new distributed_agent_api_1.DistributedAgentApi(options, runGroup, agentName);
        createAgentLockfileAndSetUpListeners(api, options, agentName);
        return executeTasks(options, api)
            .then((res) => __awaiter(this, void 0, void 0, function* () {
            yield (0, metric_logger_1.submitRunMetrics)(options);
            return res;
        }))
            .catch((e) => __awaiter(this, void 0, void 0, function* () {
            yield api.completeRunGroupWithError(`Critical Error in Agent: "${e.message}"`);
            throw e;
        }));
    });
}
exports.startAgent = startAgent;
//# sourceMappingURL=distributed-agent.impl.js.map