"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.nxCloudDistributedTasksRunner = void 0;
const fs_1 = require("fs");
const stripJsonComments = require("strip-json-comments");
const create_unchanged_value_timeout_1 = require("../../../utilities/create-unchanged-value-timeout");
const environment_1 = require("../../../utilities/environment");
const metric_logger_1 = require("../../../utilities/metric-logger");
const serializer_overrides_1 = require("../../../utilities/serializer-overrides");
const waiter_1 = require("../../../utilities/waiter");
const error_reporter_api_1 = require("../../api/error-reporter.api");
const print_run_group_error_1 = require("../../error/print-run-group-error");
const e2e_encryption_1 = require("../../file-storage/e2e-encryption");
const file_storage_1 = require("../../file-storage/file-storage");
const distributed_execution_api_1 = require("./distributed-execution.api");
const split_task_graph_into_stages_1 = require("./split-task-graph-into-stages");
const task_graph_creator_1 = require("./task-graph-creator");
const { output, workspaceRoot, getOutputs, Cache, } = require('../../../utilities/nx-imports');
class NoopLifeCycle {
    scheduleTask(task) { }
    startTask(task) { }
    endTask(task, code) { }
}
function runDistributedExecution(api, options, context, fileStorage, cache, runGroup, taskGraph) {
    return __awaiter(this, void 0, void 0, function* () {
        const id = yield api.start((0, distributed_execution_api_1.createStartRequest)(runGroup, (0, split_task_graph_into_stages_1.splitTasksIntoStages)(taskGraph), options));
        return yield processTasks(api, fileStorage, cache, options, id, Object.values(taskGraph.tasks), context);
    });
}
function processTasks(api, fileStorage, cache, options, executionId, tasks, context) {
    return __awaiter(this, void 0, void 0, function* () {
        const processedTasks = {};
        const failIfNumberOfCompletedTasksDoesNotChangeAfterTimeout = (0, create_unchanged_value_timeout_1.createUnchangedValueTimeout)({
            title: `No new completed tasks after ${environment_1.NO_COMPLETED_TASKS_TIMEOUT / 1000} seconds.`,
            timeout: environment_1.NO_COMPLETED_TASKS_TIMEOUT,
        });
        const waiter = new waiter_1.Waiter();
        while (true) {
            if (environment_1.VERBOSE_LOGGING) {
                output.note({
                    title: 'Waiting...',
                });
            }
            yield waiter.wait();
            const r = yield api.status(executionId);
            if (environment_1.VERBOSE_LOGGING) {
                output.note({
                    title: `Status update`,
                    bodyLines: [
                        `executionId: ${executionId}`,
                        `executionStatus: ${r.executionStatus}`,
                        `number of completed tasks: ${r.completedTasks.length}`,
                        `error: ${r.criticalErrorMessage}`,
                    ],
                });
            }
            if (r.criticalErrorMessage) {
                output.error({
                    title: 'Distributed Execution Terminated',
                    bodyLines: ['Error:', r.criticalErrorMessage],
                });
                process.exit(1);
            }
            failIfNumberOfCompletedTasksDoesNotChangeAfterTimeout(r.completedTasks.length);
            for (const t of r.completedTasks) {
                if (processedTasks[t.taskId])
                    continue;
                yield processTask(fileStorage, cache, context, options, tasks, t);
                waiter.reset();
                processedTasks[t.taskId] = true;
            }
            if (r.executionStatus === 'COMPLETED') {
                return { commandStatus: r.commandStatus, runUrl: r.runUrl };
            }
        }
    });
}
function processTask(fileStorage, cache, context, options, tasks, completedTask) {
    return __awaiter(this, void 0, void 0, function* () {
        if (environment_1.VERBOSE_LOGGING) {
            output.note({
                title: `Processing task ${completedTask.taskId}`,
            });
        }
        const matchingTask = tasks.find((tt) => completedTask.taskId === tt.id);
        if (!matchingTask) {
            throw new Error(`Found unknown task: ${completedTask.taskId}`);
        }
        if (environment_1.VERBOSE_LOGGING) {
            output.note({
                title: `Retrieving artifacts from ${completedTask.url}`,
            });
        }
        yield fileStorage.retrieve(completedTask.hash, completedTask.url, options.cacheDirectory || './node_modules/.cache/nx');
        const cachedResult = yield cache.get(Object.assign(Object.assign({}, matchingTask), { hash: completedTask.hash }));
        const outputs = getOutputs(context.projectGraph.nodes, matchingTask);
        if (environment_1.VERBOSE_LOGGING) {
            output.note({
                title: `Extracting artifacts`,
                bodyLines: outputs,
            });
        }
        yield cache.copyFilesFromCache(completedTask.hash, cachedResult, outputs);
        output.logCommand(getCommand(matchingTask));
        process.stdout.write(cachedResult.terminalOutput);
        output.addVerticalSeparator();
    });
}
function getCommand(task) {
    const config = task.target.configuration
        ? `:${task.target.configuration}`
        : '';
    return [
        'nx',
        'run',
        `${task.target.project}:${task.target.target}${config}`,
        (0, serializer_overrides_1.serializeOverrides)(task),
    ].join(' ');
}
function attachSignalListenersToCompleteRunGroupOnError(api, runGroup) {
    process.on('SIGINT', () => __awaiter(this, void 0, void 0, function* () {
        yield api.completeRunGroupWithError(runGroup, 'Main job was terminated via SIGINT');
        process.exit(1);
    }));
    process.on('SIGTERM', () => __awaiter(this, void 0, void 0, function* () {
        yield api.completeRunGroupWithError(runGroup, 'Main job was terminated via SIGTERM');
        process.exit(1);
    }));
}
const nxCloudDistributedTasksRunner = (tasks, options, context) => __awaiter(void 0, void 0, void 0, function* () {
    if (environment_1.VERBOSE_LOGGING) {
        output.note({
            title: 'Starting distributed command execution',
        });
    }
    options.lifeCycle = new NoopLifeCycle();
    const runGroup = (0, environment_1.getRunGroup)();
    if (!runGroup) {
        (0, print_run_group_error_1.printRunGroupError)();
        return process.exit(1);
    }
    const encryption = new e2e_encryption_1.E2EEncryption(environment_1.ENCRYPTION_KEY || options.encryptionKey);
    const errorReporter = new error_reporter_api_1.ErrorReporterApi(options);
    const fileStorage = new file_storage_1.FileStorage(encryption, errorReporter, true);
    const cache = new Cache(options);
    const api = new distributed_execution_api_1.DistributedExecutionApi(options);
    attachSignalListenersToCompleteRunGroupOnError(api, runGroup);
    try {
        const taskGraph = getTaskGraph(context, tasks, options);
        const r = yield runDistributedExecution(api, options, context, fileStorage, cache, runGroup, taskGraph);
        if (r.commandStatus === 0) {
            output.success({
                title: 'Successfully completed running the command.',
                bodyLines: [`See run details at ${r.runUrl}`],
            });
        }
        else {
            output.error({
                title: 'Command execution failed.',
                bodyLines: [`See run details at ${r.runUrl}`],
            });
        }
        yield (0, metric_logger_1.submitRunMetrics)(options);
        process.exit(r.commandStatus);
    }
    catch (e) {
        output.error({
            title: 'Unable to complete a run.',
            bodyLines: [e.message],
        });
        if (e.axiosException) {
            console.log(e.axiosException);
        }
        else {
            console.log(e);
        }
        try {
            yield api.completeRunGroupWithError(runGroup, `Main job terminated with an error: "${e.message}"`);
        }
        finally {
            process.exit(1);
        }
    }
});
exports.nxCloudDistributedTasksRunner = nxCloudDistributedTasksRunner;
function getTaskGraph(context, tasks, options) {
    if (context.taskGraph) {
        return context.taskGraph;
    }
    else {
        const nxjson = JSON.parse(stripJsonComments((0, fs_1.readFileSync)(`${workspaceRoot}/nx.json`).toString()));
        return new task_graph_creator_1.TaskGraphCreator(context.projectGraph, getDefaultDependencyConfigs(nxjson, options)).createTaskGraph(tasks);
    }
}
function getDefaultDependencyConfigs(nxJson, runnerOptions) {
    var _a, _b;
    const defaults = (_a = nxJson.targetDependencies) !== null && _a !== void 0 ? _a : {};
    const strictlyOrderedTargets = runnerOptions
        ? (_b = runnerOptions.strictlyOrderedTargets) !== null && _b !== void 0 ? _b : ['build']
        : [];
    // Strictly Ordered Targets depend on their dependencies
    for (const target of strictlyOrderedTargets) {
        defaults[target] = defaults[target] || [];
        defaults[target].push({
            target,
            projects: 'dependencies',
        });
    }
    return defaults;
}
//# sourceMappingURL=distributed-execution.runner.js.map